/*  Copyright (C) 2008 Joshua Judson Rosen <rozzin@geekspace.com>.

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 3 as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING. If not, see
    <http://www.gnu.org/licenses/> or write to:

        The Free Software Foundation, inc.
        51 Franklin Street, Fifth Floor
        Boston, MA 02110-1301
        USA
*/

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include <glib.h>

int
hash_filename (const char *path)
{
  unsigned int hash; /* for seeding srand() */
  char *name = g_path_get_basename (path);
  char hash_buf[(sizeof hash) * 2 + 1] = "";
  int slave_input[2]; FILE *slave_input_stream;
  int slave_output[2]; FILE *slave_output_stream;

  pipe (slave_input);
  pipe (slave_output);

  switch (fork ()) {
  case -1:
    return -1;

  case 0: /* child */
    dup2 (slave_input[0], 0); close (slave_input[0]); close (slave_input[1]);
    dup2 (slave_output[1], 1);  close (slave_output[1]);
    execlp ("md5sum", "md5sum", NULL);
    exit (1);
    break;

  default: /* parent */
    {
      slave_input_stream = fdopen (slave_input[1], "w");
      slave_output_stream = fdopen (slave_output[0], "r");

      fprintf (slave_input_stream, "%s", name);
      fflush (slave_input_stream);
      fclose (slave_input_stream); // close (slave_input[1]);

      read (slave_output[0], hash_buf, (sizeof hash_buf)-1);
      hash_buf[(sizeof hash_buf)-1] = '\0';
      sscanf (hash_buf, "%x", &hash);
      break;
    }
  }

  g_free (name);
  return hash;
}
